@@ -7,8 +7,9 @@
  * See the COPYING file for the terms of usage and distribution.
  */
 
-#include "log4cpp/Portability.hh"
-#include "log4cpp/NDC.hh"
+#include "PortabilityImpl.hh"
+#include <log4cpp/NDC.hh>
+#include <log4cpp/threading/Threading.hh>
 
 namespace log4cpp {
 
@@ -20,10 +21,12 @@ namespace log4cpp {
     NDC::DiagnosticContext::DiagnosticContext(const std::string& message, 
             const DiagnosticContext& parent) :
         message(message),
-        fullMessage(parent.message + " " + message) {
+        fullMessage(parent.fullMessage + " " + message) {
     }
 
-    NDC NDC::_nDC;
+    namespace {
+        threading::ThreadLocalDataHolder<NDC> _nDC;
+    }
 
     void NDC::clear() {
         getNDC()._clear();
@@ -58,7 +61,14 @@ namespace log4cpp {
     }
 
     NDC& NDC::getNDC() {
-        return _nDC;
+        NDC* nDC = _nDC.get();
+
+        if (!nDC) {
+            nDC = new NDC();
+            _nDC.reset(nDC);
+        }
+
+        return *nDC;
     }
 
     NDC::NDC() {
@@ -78,9 +88,7 @@ namespace log4cpp {
     const std::string& NDC::_get() const {
         static std::string empty = "";
 
-        return _stack.empty() ?
-            empty :
-            _stack.back().fullMessage;
+        return (_stack.empty() ? empty : _stack.back().fullMessage);
     }
 
     int NDC::_getDepth() const {
